{% include "documents/_modals_document_upload_new_version.html" %}
{% include "documents/_modals_document_send_by_email.html" %}
{% include "documents/_modals_document_delete.html" %}
{% include "documents/_modals_document_edit.html" %}
